#ifndef _CAMERA_H_
#define _CAMERA_H_

#include "vector3.h"

class Camera
{
    Vector3 m_eye;
    Vector3 m_lookAt;
    Vector3 m_up;

    double m_pitch;
    double m_yaw;

    public:
    Camera(void);
    Camera(const Vector3 &eye, const Vector3 &view);
    
    void setEye(const Vector3 &eye);

    /* basis vectors */
    Vector3 eye(void) const { return m_eye; }
    Vector3 lookAt(void) const { return m_lookAt; }
    Vector3 up(void) const { return m_up; }

    /* ortho-normal vectors */
    Vector3 view(void) const { return m_lookAt - m_eye; }
    Vector3 right(void) const { return view().cross(m_up); }
    Vector3 trueUp(void) const { return right().cross(view()); }

    /* transformations */
    void pitch(double degrees);
    void yaw(double degrees);
    void translate(const Vector3 &t);
    void moveForward(double d);
    void strafe(double d);

    private:
    void updateVectors(void);
    void updatePitchAndYaw(void);
};

#endif
