#include "matrix.h"

#include <GL/glut.h>

Matrix::Matrix(void)
{
    for (int i = 0; i < 16; i++) {
        m_matrix[i] = i % 5 ? 0.0f : 1.0f;
    }
}

Matrix Matrix::identity(void)
{
    return Matrix();
}

Matrix Matrix::fromRotation(double degrees, const Vector3 &v)
{
    Matrix m;

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    {
        glLoadIdentity();
        glRotatef(degrees, v.x(), v.y(), v.z());
        glGetFloatv(GL_MODELVIEW_MATRIX, m.m_matrix);
    }
    glPopMatrix();

    return m;
}

Vector3 Matrix::operator*(const Vector3 &rhs) const
{
    float vector[4] = {rhs.x(), rhs.y(), rhs.z(), 1.0};

    for (int i = 0; i < 4; i++) {
        for (int j = 0; j < 4; j++) {
            int idx = i * 4 + j;
            vector[j] += m_matrix[idx] * vector[i];
        }
    }

    return Vector3(vector[0], vector[1], vector[2]);
}

Matrix &Matrix::operator*=(const Matrix &rhs)
{
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    {
        glLoadIdentity();
        glMultMatrixf(rhs.m_matrix);
        glMultMatrixf(m_matrix);
        glGetFloatv(GL_MODELVIEW_MATRIX, m_matrix);
    }
    glPopMatrix();

    return *this;
}
