#ifndef _MOVABLE_H_
#define _MOVABLE_H_

#include "vector3.h"
#include "matrix.h"

class Movable
{
    Vector3 m_translate;
    float m_scale;
    Matrix m_rotate;

    Vector3 m_velocity;

    public:
        Movable(const Vector3 &position, const Vector3 &velocity=Vector3(0.0, 0.0, 0.0)) :
            m_translate(position), m_scale(1.0), m_rotate(Matrix::identity()),
            m_velocity(velocity)
        {}

        Movable(double x, double y, double z) :
            m_translate(x, y, z), m_scale(1.0), m_rotate(Matrix::identity()),
            m_velocity(0.0, 0.0, 0.0)
        {}

        Movable(void) :
            m_translate(0.0, 0.0, 0.0), m_scale(1.0), m_rotate(Matrix::identity()),
            m_velocity(0.0, 0.0, 0.0)
        {}

        void doTransforms(void) const;
        void doMovement(double timestep);

        Vector3 location(void) const { return m_translate; }
        Vector3 velocity(void) const { return m_velocity; }
        Vector3 forward(void) const  { return m_rotate * Vector3(1.0, 0.0, 0.0); }

        void scale(float s);
        void translate(float tx, float ty, float tz);
        void rotate(float deg, Vector3 v);

        void setLocation(double x, double y, double z) { m_translate.set(x, y, z); }
        void setRotation(double deg, const Vector3 &v);

        void setVelocity(double vx, double vy, double vz) { setVelocity(Vector3(vx, vy, vz)); }
        void setVelocity(const Vector3 &v) { m_velocity = v; }
};

#endif
