#ifndef _VECTOR3_H_
#define _VECTOR3_H_

#include <cmath>

class Vector3
{
    double m_x, m_y, m_z;

    public:
    Vector3(double x, double y, double z) : m_x(x), m_y(y), m_z(z) {}
    Vector3(void) : m_x(0.0), m_y(0.0), m_z(0.0) {}

    void set(double x, double y, double z) { m_x = x; m_y = y; m_z = z; }
    void setX(double x) { m_x = x; }
    void setY(double y) { m_y = y; }
    void setZ(double z) { m_z = z; }

    double x(void) const { return m_x; }
    double y(void) const { return m_y; }
    double z(void) const { return m_z; }

    double lenSq(void) const    { return m_x*m_x + m_y*m_y + m_z*m_z; }
    double len(void) const      { return sqrt(lenSq()); }

    Vector3 &normalize(void)
    {
        return operator/=(len());
    }

    Vector3 unit(void) const
    {
        double l = len();
        return Vector3(m_x/l, m_y/l, m_z/l);
    }

    double dot(const Vector3 &v) const
    {
        return m_x*v.m_x + m_y*v.m_y + m_z*v.m_z;
    }

    Vector3 cross(const Vector3 &v) const
    {
        return Vector3(
                m_y*v.m_z - m_z*v.m_y,
                m_z*v.m_x - m_x*v.m_z,
                m_x*v.m_y - m_y*v.m_x
            );
    }

    Vector3 operator+(const Vector3 &v) const
    {
        return Vector3(m_x+v.m_x, m_y+v.m_y, m_z+v.m_z);
    }

    Vector3 operator-(const Vector3 &v) const
    {
        return Vector3(m_x-v.m_x, m_y-v.m_y, m_z-v.m_z);
    }

    Vector3 operator*(double s) const
    {
        return Vector3(m_x*s, m_y*s, m_z*s);
    }

    Vector3 operator/(double s) const
    {
        return Vector3(m_x/s, m_y/s, m_z/s);
    }

    Vector3 &operator+=(const Vector3 &v)
    {
        m_x += v.m_x;
        m_y += v.m_y;
        m_z += v.m_z;
        return *this;
    }

    Vector3 &operator-=(const Vector3 &v)
    {
        m_x -= v.m_x;
        m_y -= v.m_y;
        m_z -= v.m_z;
        return *this;
    }

    Vector3 &operator*=(double s)
    {
        m_x *= s;
        m_y *= s;
        m_z *= s;
        return *this;
    }

    Vector3 &operator/=(double s)
    {
        m_x /= s;
        m_y /= s;
        m_z /= s;
        return *this;
    }
};

#endif
