#include "viewport.h"

void Viewport::set(int width, int height)
{
    m_width = width;
    m_height = height;
    m_right = 2.0f * (float)width/height;
    m_left = -m_right;
}

Vector Viewport::p2i(int x, int y) const
{
    double g = m_width / 2.0f;
    double h = (m_width - 1) / 2.0f;
    double ix = (x - h) / g;

    double i = m_height / 2.0f;
    double j = (m_height - 1) / 2.0f;
    double iy = (m_height - 1 - y - j) / i;

    return Vector(ix, iy);
}

Vector Viewport::i2w(Vector i) const
{
    double c = 2.0f / (m_right - m_left);
    double d = (-m_right- m_left) / (m_right - m_left);
    double wx = (i.x - d) / c;

    double e = 2.0f / (m_top - m_bottom);
    double f = (-m_top - m_bottom) / (m_top - m_bottom);
    double wy = (i.y - f) / e;

    return Vector(wx, wy);
}

Vector3 Viewport::trackball(int x, int y) const
{
    Vector vi = p2i(x, y);
    double rsqr = vi.LenSquare();

    if (rsqr <= 1.0) {
        double z = sqrt(1.0 - rsqr);
        return Vector3(vi.x, vi.y, z);
    }

    double len = sqrt(rsqr);
    return Vector3(vi.x/len, vi.y/len, 0.0);
}
